@@ -14,6 +14,7 @@ import cdb_query_archive_parsers
 import cdb_query_archive_class
 
 import netcdf_utils
+import certificates
 
 def main_CMIP5():
     main('CMIP5')
@@ -28,6 +29,7 @@ def main(project):
     import textwrap
 
     #Option parser
+    version_num='1.0.3'
     description=textwrap.dedent('''\
     This script queries a {0} archive. It can query:
     1. a local POSIX-based archive that follows the {0} DRS
@@ -35,8 +37,11 @@ def main(project):
     2. the ESGF {0} archive.
 
     '''.format(project))
-    epilog='Frederic Laliberte, Paul Kushner 06/2014'
-    version_num='1.0.0'
+    epilog='Version {0}: Frederic Laliberte, Paul Kushner 08/2014\n\
+\n\
+If using this code to retrieve and process data from the ESGF please cite:\n\n\
+Efficient, robust and timely analysis of Earth System Models: a database-query approach (2014):\n\
+F. Laliberté, Juckes, M., Denvil, S., Kushner, P. J., Bull. Amer. Meteor. Soc., Submitted.'.format(version_num)
     parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                             description=description,
                             version='%(prog)s '+version_num,
@@ -68,6 +73,9 @@ def main(project):
         netcdf_utils.apply(options,project_drs)
     elif options.command=='convert':
         netcdf_utils.convert(options,project_drs)
+    elif options.command=='certificates':
+        certificates.retrieve_certificates(options.username,options.password,options.registering_service)
+        #certificates.test_certificates()
     elif 'in_diagnostic_headers_file' in dir(options):
         paths_dict=cdb_query_archive_class.SimpleTree(options,project_drs)
         #Run the command:
