@@ -41,6 +41,15 @@ def excluded_slicing_arguments(parser,project_drs,exclude_args=[],action_type='s
                                 )
     return
 
+def complex_slicing(parser,project_drs,action_type='store'):
+    complex_query_type=(lambda x: tuple(x.split(',')))
+    parser.add_argument('--complex_query',action=action_type,type=complex_query_type,
+                        help='Complex inclusion with fields specified by --field',
+                        default=[])
+    parser.add_argument('--Xcomplex_query',action=action_type, type=complex_query_type,
+                        help='Complex exclusion with fields specified by --field.',
+                        default=[])
+    return
 
 def generate_subparsers(parser,epilog,project_drs):
     #Discover tree
@@ -55,6 +64,7 @@ def generate_subparsers(parser,epilog,project_drs):
     download_raw(subparsers,epilog,project_drs)
     apply(subparsers,epilog,project_drs)
     convert(subparsers,epilog,project_drs)
+    certificates(subparsers,epilog,project_drs)
     return
 
 #def quick_ask(subparsers,epilog,project_drs):
@@ -95,6 +105,11 @@ def ask(subparsers,epilog,project_drs):
                                                  to function properly. If it fails it is possible the servers are down.'''),
                                            epilog=epilog_ask
                                          )
+    parser.add_argument('--related_experiments',
+                                 default=False, action='store_true',
+                                 help='When this option is activated, queried experiments are assumed to be related.\n\
+                                       In this situation, cdb_query will discard ({0}) tuples that do not have variables for\n\
+                                       ALL of the requested experiments'.format(','.join(project_drs.simulations_desc)))
     input_arguments_json(parser)
     output_arguments(parser)
     parser.add_argument('--list_only_field',default=None, choices=project_drs.remote_fields,
@@ -139,6 +154,8 @@ def list_fields(subparsers,epilog,project_drs):
     slicing_arguments(inc_group,project_drs,action_type='append')
     exc_group = parser.add_argument_group('Exclusions')
     excluded_slicing_arguments(exc_group,project_drs,action_type='append')
+    comp_group = parser.add_argument_group('Complex Query')
+    complex_slicing(comp_group,project_drs,action_type='append')
     return
 
 def validate(subparsers,epilog,project_drs):
@@ -155,6 +172,11 @@ def validate(subparsers,epilog,project_drs):
                                          certificates have not been installed on this machine.'),
                                    epilog=epilog_validate,
                                  )
+    parser.add_argument('--related_experiments',
+                                 default=False, action='store_true',
+                                 help='When this option is activated, queried experiments are assumed to be related.\n\
+                                       In this situation, cdb_query will discard ({0}) tuples that do not have variables for\n\
+                                       ALL of the requested experiments'.format(','.join(project_drs.simulations_desc)))
     input_arguments(parser)
     output_arguments(parser)
     proc_group = parser.add_argument_group('These arguments set threading options')
@@ -176,6 +198,10 @@ def validate(subparsers,epilog,project_drs):
     slicing_arguments(inc_group,project_drs,action_type='append')
     exc_group = parser.add_argument_group('Exclusions')
     excluded_slicing_arguments(exc_group,project_drs,action_type='append')
+    comp_group = parser.add_argument_group('Complex Query')
+    comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
+                                       help='List the field (or fields if repeated) found in the file' )
+    complex_slicing(comp_group,project_drs,action_type='append')
     return
 
 def download(subparsers,epilog,project_drs):
@@ -209,6 +235,10 @@ def download(subparsers,epilog,project_drs):
     slicing_arguments(inc_group,project_drs,action_type='append')
     exc_group = parser.add_argument_group('Exclusions')
     excluded_slicing_arguments(exc_group,project_drs,action_type='append')
+    comp_group = parser.add_argument_group('Complex Query')
+    comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
+                                       help='List the field (or fields if repeated) found in the file' )
+    complex_slicing(comp_group,project_drs,action_type='append')
 
     data_node_group = parser.add_argument_group('Limit download from specific data nodes')
     data_node_group.add_argument('--data_node',type=str,action='append',help='Retrieve only from the specified data nodes')
@@ -255,6 +285,10 @@ def download_raw(subparsers,epilog,project_drs):
     slicing_arguments(inc_group,project_drs,action_type='append')
     exc_group = parser.add_argument_group('Exclusions')
     excluded_slicing_arguments(exc_group,project_drs,action_type='append')
+    comp_group = parser.add_argument_group('Complex Query')
+    comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
+                                       help='List the field (or fields if repeated) found in the file' )
+    complex_slicing(comp_group,project_drs,action_type='append')
     return
 
 def convert(subparsers,epilog,project_drs):
@@ -275,6 +309,10 @@ def convert(subparsers,epilog,project_drs):
     slicing_arguments(inc_group,project_drs)
     exc_group = parser.add_argument_group('Exclusions')
     excluded_slicing_arguments(exc_group,project_drs)
+    comp_group = parser.add_argument_group('Complex Query')
+    comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
+                                       help='List the field (or fields if repeated) found in the file' )
+    complex_slicing(comp_group,project_drs,action_type='append')
     return
 
 
@@ -294,7 +332,7 @@ def apply(subparsers,epilog,project_drs):
 
     select_group = parser.add_argument_group('These arguments specify the structure of the output')
     select_group.add_argument('--add_fixed',default=False, action='store_true',help='include fixed variables')
-    select_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.official_drs_no_version,
+    select_group.add_argument('-k','--keep_field',action='append', type=str, choices=project_drs.official_drs_no_version,
                                        help='Keep these fields in the applied file.' )
     proc_group = parser.add_argument_group('These arguments set threading options')
     proc_group.add_argument('--num_procs',
@@ -308,5 +346,21 @@ def apply(subparsers,epilog,project_drs):
     slicing_arguments(inc_group,project_drs,action_type='append')
     exc_group = parser.add_argument_group('Exclusions')
     excluded_slicing_arguments(exc_group,project_drs,action_type='append')
+    comp_group = parser.add_argument_group('Complex Query')
+    comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.official_drs_no_version,
+                                       help='Complex queyr fields.' )
+    complex_slicing(comp_group,project_drs,action_type='append')
     return
+
+def certificates(subparsers,epilog,project_drs):
+    epilog_certificates=textwrap.dedent(epilog)
+    parser=subparsers.add_parser('certificates',
+                                           description=textwrap.dedent('Recovers ESGF certificates'),
+                                           epilog=epilog_certificates
+                                         )
+    parser.add_argument('username',help="Username")
+    parser.add_argument('password',help="Password")
+    parser.add_argument('registering_service',help="Registering service",choices=['badc','jpl'])
+    return
+
     
