@@ -122,7 +122,9 @@ class soft_links:
                     if self.paths_ordering['checksum'][file_id]==self.paths_ordering['checksum'][equivalent_file_id]:
                         paths_id_list.append(file_id)
         #Sort paths_ordering:
-        self.paths_ordering=self.paths_ordering[np.sort(paths_id_list)]
+        if len(paths_id_list)>0:
+            self.paths_ordering=self.paths_ordering[np.sort(paths_id_list)]
+            
         #Finally, set the path_id field to be following the indices in paths_ordering:
         self.paths_ordering['path_id']=range(len(self.paths_ordering))
 
@@ -143,7 +145,7 @@ class soft_links:
         #Include a filter on years: 
         time_desc={}
         years_range=range(*years)
-        years_range+=[years_range[-1]+1]
+        years_range.append(years[-1])
         if years[0]<10:
             #This is important for piControl
             years_range=list(np.array(years_range)+np.min([date.year for date in time_axis_unique_date]))
