@@ -13,21 +13,23 @@ import sqlalchemy
 
 import numpy as np
 
+import remote_netcdf
+
 
 queryable_file_types=['HTTPServer','local_file']
 
-def find_time(pointers,file_expt):
+def find_time(pointers,file_expt,semaphores=None):
     #session,file_expt,path_name,file_type,propagated_values):
     #Top function to define how the time axis is created:
     if file_expt.file_type in queryable_file_types:
-        find_time_file(pointers,file_expt)
+        find_time_file(pointers,file_expt,semaphores=semaphores)
     #elif file_expt.file_type in ['GridFTP']:
     #    find_time_file(pointers,file_expt)
     #elif file_expt.file_type in ['local_file','OPeNDAP']:
     #    find_time_opendap(pointers,file_expt)
     return
         
-def find_time_file(pointers,file_expt):#session,file_expt,path_name):
+def find_time_file(pointers,file_expt,semaphores=None):#session,file_expt,path_name):
     #If the path is a remote file, we must use the time stamp
     filename=os.path.basename(file_expt.path)
     
@@ -40,6 +42,10 @@ def find_time_file(pointers,file_expt):#session,file_expt,path_name):
         time_stamp=filename.replace('.nc','').split('_')[-1].split('|')[0]
     #time_stamp[0] == 'r':
 
+    years_requested=[int(year) for year in pointers.header['experiment_list'][file_expt.experiment].split(',')]
+    years_list_requested=range(*years_requested)
+    years_list_requested.append(years_requested[1])
+
     #Recover date range from filename:
     years_range = [int(date[:4]) for date in time_stamp.split('-')]
     #Check for yearly data
@@ -50,26 +56,44 @@ def find_time_file(pointers,file_expt):#session,file_expt,path_name):
     years_list=range(*years_range)
     years_list.append(years_range[1])
 
+    years_list=[ year for year in years_list if year in years_list_requested]
+
     #Record in the database:
     if file_expt.file_type in ['local_file']:
         file_available=True
+        file_queryable=True
     else:
         file_available = retrieval_utils.check_file_availability(file_expt.path.split('|')[0])
+        if file_available:
+            remote_data=remote_netcdf.remote_netCDF(file_expt.path.split('|')[0].replace('fileServer','dodsC'),semaphores)
+            file_queryable=remote_data.is_available()
     for year in years_list:
         for month in range(1,13):
             if not ( (year==years_range[0] and month<months_range[0]) or
                      (year==years_range[1] and month>months_range[1])   ):
-                if file_available:
+                #Record checksum of local files:
+                if file_expt.file_type in ['local_file'] and len(file_expt.path.split('|')[1])==0:
+                    #Record checksum
+                    file_expt.path+=retrieval_utils.md5_for_file(open(file_expt.path.split('|')[0],'r'))
+
+                if file_available and file_queryable:
+                    #If file is avaible and queryable, keep it:
                     file_expt_copy = copy.deepcopy(file_expt)
                     setattr(file_expt_copy,'time',str(year)+str(month).zfill(2))
                     pointers.session.add(file_expt_copy)
                     pointers.session.commit()
+                elif file_available:
+                    #If file is avaible but queryable, broadcast it:
+                    print 'Path {0} is available but does not have OPeNDAP services enabled.'.format(file_expt.path)
+                    #Do not broadcast it later:
+                    file_available=False
     return
 
 def obtain_time_list(diagnostic,project_drs,var_name,experiment,model):
     #Do this without fx variables:
-    conditions=([
-                 nc_Database.File_Expt.var==var_name,] +
+    conditions=(
+                 [nc_Database.File_Expt.var==var_name,] +
+                 [nc_Database.File_Expt.experiment==experiment,] +
                [ getattr(nc_Database.File_Expt,desc)==model[desc_id]
                  for desc_id,desc in enumerate(project_drs.simulations_desc)]
                )
@@ -80,7 +104,8 @@ def obtain_time_list(diagnostic,project_drs,var_name,experiment,model):
                             ).filter(sqlalchemy.and_(*conditions)).distinct().all()]
     return time_list_var
 
-def find_model_list(diagnostic,project_drs,model_list,experiment,min_time):
+#def find_model_list(diagnostic,project_drs,model_list,experiment,min_time):
+def find_model_list(diagnostic,project_drs,model_list,experiment):
     period_list = diagnostic.header['experiment_list'][experiment]
     if not isinstance(period_list,list): period_list=[period_list]
     years_list=[]
@@ -93,7 +118,6 @@ def find_model_list(diagnostic,project_drs,model_list,experiment,min_time):
         for month in get_diag_months_list(diagnostic):
             time_list.append(str(year).zfill(4)+str(month).zfill(2))
 
-    #Remove the fixed variables manually:
     model_list_var=copy.copy(model_list)
 
     for model in model_list_var:
@@ -106,14 +130,17 @@ def find_model_list(diagnostic,project_drs,model_list,experiment,min_time):
                     time_list_var=obtain_time_list(diagnostic,project_drs,var_name,experiment,model)
                     if len(time_list_var)>0:
                         min_time_list.append(int(np.floor(np.min([int(time) for time in time_list_var])/100.0)*100))
-            min_time['_'.join(model)+'_'+experiment]=np.min(min_time_list)
+            #min_time['_'.join(model)+'_'+experiment]=np.min(min_time_list)
+            min_time=np.min(min_time_list)
         else:
-            min_time['_'.join(model)+'_'+experiment]=0
+            #min_time['_'.join(model)+'_'+experiment]=0
+            min_time=0
 
         for var_name in diagnostic.header['variable_list'].keys():
             if not diagnostic.header['variable_list'][var_name][0] in ['fx','clim']:
                 time_list_var=obtain_time_list(diagnostic,project_drs,var_name,experiment,model)
-                time_list_var=[str(int(time)-int(min_time['_'.join(model)+'_'+experiment])).zfill(6) for time in time_list_var]
+                #time_list_var=[str(int(time)-int(min_time['_'.join(model)+'_'+experiment])).zfill(6) for time in time_list_var]
+                time_list_var=[str(int(time)-int(min_time)).zfill(6) for time in time_list_var]
                 if not set(time_list).issubset(time_list_var):
                     missing_vars.append(var_name+':'+','.join(
                                         diagnostic.header['variable_list'][var_name])+
@@ -123,10 +150,16 @@ def find_model_list(diagnostic,project_drs,model_list,experiment,min_time):
                                        )
         if len(missing_vars)>0:
            #print('\nThe reasons why some simulations were excluded:')
-           print('_'.join(model)+' excluded because experiment '+experiment+' is missing variables:\n'),
-           for item in missing_vars: print(item)
+           if 'experiment' in project_drs.simulations_desc:
+               if experiment==model[project_drs.simulations_desc.index('experiment')]:
+                   print('_'.join(model)+' excluded because it is missing variables:\n'),
+                   for item in missing_vars: print(item)
+           else:
+               print('_'.join(model)+' excluded because experiment '+experiment+' is missing variables:\n'),
+               for item in missing_vars: print(item)
            model_list.remove(model)
-    return model_list, min_time
+    #return model_list, min_time
+    return model_list
 
 def get_diag_months_list(diagnostic):
     if 'months_list' in diagnostic.header.keys():
@@ -142,13 +175,12 @@ def optimset_distributed(database,options,semaphores):
     return filepath
 
 def optimset(database,options,semaphores=None):
-    database.load_database(options,find_time)
+    database.load_database(options,find_time,semaphores=semaphores)
     #Find the list of institute / model with all the months for all the years / experiments and variables requested:
     intersection(database,options)
     
-    dataset=database.nc_Database.write_database(database.header,options,'record_meta_data',semaphores=semaphores)
+    dataset, output=database.nc_Database.write_database(database.header,options,'record_meta_data',semaphores=semaphores)
     database.close_database()
-    output=dataset.filepath()
     dataset.close()
     return output
 
@@ -162,10 +194,13 @@ def intersection(database,options):
                                 simulation[database.drs.simulations_desc.index('ensemble')]!='r0i0p0']
     model_list=copy.copy(simulations_list_no_fx)
 
-    min_time=dict()
-        #print experiment,model_list
-    for experiment in database.header['experiment_list'].keys():
-        model_list, min_time = find_model_list(database,database.drs,model_list,experiment,min_time)
+    if not 'experiment' in database.drs.simulations_desc:
+        for experiment in database.header['experiment_list'].keys():
+            model_list = find_model_list(database,database.drs,model_list,experiment)
+        model_list_combined=model_list
+    else:
+        model_list_combined=set().union(*[find_model_list(database,database.drs,model_list,experiment) 
+                                           for experiment in database.header['experiment_list'].keys()])
 
     #Step two: create the new paths dictionary:
     variable_list_requested=[]
@@ -173,13 +208,29 @@ def intersection(database,options):
         variable_list_requested.append((var_name,)+tuple(database.header['variable_list'][var_name]))
 
     #Step three: find the models to remove:
-    simulations_desc_indices_without_ensemble=range(0,len(database.drs.simulations_desc))
-    simulations_desc_indices_without_ensemble.remove(database.drs.simulations_desc.index('ensemble'))
-    models_to_remove=set(simulations_list_no_fx).difference(model_list)
+    models_to_remove=set(simulations_list_no_fx).difference(model_list_combined)
 
     #Step four: remove from database:
     for model in models_to_remove:
-        conditions=[ getattr(nc_Database.File_Expt,field)==model[field_id] for field_id, field in enumerate(itemgetter(*simulations_desc_indices_without_ensemble)(database.drs.simulations_desc))]
+        conditions=[getattr(nc_Database.File_Expt,field)==model[field_id] for field_id, field in enumerate(database.drs.simulations_desc)]
         database.nc_Database.session.query(nc_Database.File_Expt).filter(*conditions).delete()
-                
+    
+    #Remove fixed variables:
+    simulations_list=database.nc_Database.simulations_list()
+    simulations_list_no_fx=[simulation for simulation in simulations_list if 
+                                simulation[database.drs.simulations_desc.index('ensemble')]!='r0i0p0']
+    models_to_remove=set(
+                        [remove_ensemble(simulation,database.drs) for simulation in simulations_list]
+                         ).difference([remove_ensemble(simulation,database.drs) for simulation in simulations_list_no_fx])
+
+    for model in models_to_remove:
+        conditions=[ getattr(nc_Database.File_Expt,field)==model[field_id] for field_id, field in enumerate(remove_ensemble(database.drs.simulations_desc,database.drs))]
+        database.nc_Database.session.query(nc_Database.File_Expt).filter(*conditions).delete()
+
     return 
+
+def remove_ensemble(simulation,project_drs):
+    simulations_desc_indices_without_ensemble=range(0,len(project_drs.simulations_desc))
+    simulations_desc_indices_without_ensemble.remove(project_drs.simulations_desc.index('ensemble'))
+    return itemgetter(*simulations_desc_indices_without_ensemble)(simulation)
+
